# -*- coding: utf-8 -*-

#sklearn version
import sklearn
print(sklearn.__version__)

#modifier le dossier de travail
import os
os.chdir("D:/DataMining/Databases_for_mining/dataset_for_soft_dev_and_comparison/outliers")

#importer les données
import pandas
cars = pandas.read_excel("cars_outliers_novelties.xlsx",sheet_name="dataset",index_col=0)

#premières lignes
print(cars.head())

#info sur les données
print(cars.info())

#graphique par paire
import seaborn as sns
sns.pairplot(cars)

#calculer et afficher les moyennes
centroid = cars.mean()
print(centroid)

#créer un jeu de données avec la moyenne
import numpy
tmp = cars.append(centroid,ignore_index=True)
tmp['id'] = numpy.zeros(tmp.shape[0])
tmp.iloc[-1,-1] = 1
print(tmp)

#graphique avec matérialisation des barycentres empiriques
sns.pairplot(tmp,hue='id')

#classe pour le calcul de la covariance empirique
from sklearn.covariance import EmpiricalCovariance
empcov = EmpiricalCovariance()

#calcul de la covariance empirique
empcov.fit(cars)

#calcul de la distance de Mahalanobis pour chaque observation
dm_emp = empcov.mahalanobis(cars)

#affichage - trié selon la distance
print(pandas.DataFrame({"Modele":cars.index,"dist":dm_emp}).sort_values(by='dist',ascending=False))

#EllipticEnveloppe
from sklearn.covariance import EllipticEnvelope
eev = EllipticEnvelope(random_state=0)

#modélisation
eev.fit(cars)

#affichage des moyennes robustes
print(pandas.DataFrame({'var':cars.columns,'location':eev.location_}))

#masque de calcul du centroïde robuste
print(eev.support_)

#véhicules exclus des calculs
print(cars.loc[eev.support_ == False,:])

#barycentre pour les véhicules inclus dans le calcul
print(cars.loc[eev.support_ == True,:].mean())

#affichage des moyennes non-robustes et robustes
tmp = cars.append(centroid,ignore_index=True)
tmp = tmp.append(pandas.Series(index=cars.columns,data=eev.location_),ignore_index=True)
tmp['id'] = numpy.zeros(tmp.shape[0])
tmp.iloc[-2,-1] = 1
tmp.iloc[-1,-1] = 2
print(tmp)

#graphique
sns.pairplot(tmp,hue='id',diag_kind='hist')

#distance de Mahalanobis
dm = eev.mahalanobis(cars)

#afficher selon la distance
print(pandas.DataFrame({"Modele":cars.index,"dist":dm}).sort_values(by='dist',ascending=False))

#valeur seuil
print(-eev.offset_)

#decision function
df = eev.decision_function(cars)

#afficher selon la decision function
print(pandas.DataFrame({"Modele":cars.index,"decision":df}).sort_values(by='decision'))

#prediction
statut = eev.predict(cars)
print(statut)

#afficher les véhicules concernés
print(cars.loc[statut == -1])

#visualisation
cars_with_statut = cars.copy()
cars_with_statut['statut'] = -statut #pour que les points atypiques en rouge soient au premier plan
sns.pairplot(cars_with_statut,hue='statut',palette={-1:'silver',1:'red'},diag_kind='hist')

#décroissance de la distance de Mahalanobis
tmp = pandas.DataFrame({"Modele":cars.index,"dist":dm}).sort_values(by='dist',ascending=False)
sns.barplot(x='dist',y='Modele',data=tmp)

#véhicules de référence
print(cars.loc[eev.support_ == True,:])

#véhicules supplémentaires
cars_supp = pandas.read_excel("cars_outliers_novelties.xlsx",sheet_name="novelty",index_col=0)
print(cars_supp)

#identifier leur positionnement
statut_supp = eev.predict(cars_supp)
print(statut_supp)

#decision function
df_supp = eev.decision_function(cars_supp)
print(pandas.DataFrame({"Modele":cars_supp.index,"decision":df_supp}).sort_values(by='decision'))

#afficher les véhicules parmi les non atypiques
tmp = cars.loc[statut == 1].append(cars_supp)
tmp['id'] = numpy.zeros(tmp.shape[0])
tmp.iloc[-4:-1,-1] = 1
tmp.iloc[-1,-1] = 2
print(tmp)

#graphique
sns.pairplot(tmp,hue='id',palette={0:'silver',1:'green',2:'red'},diag_kind='hist')

#réduction des variables
from sklearn.preprocessing import StandardScaler
stds = StandardScaler(with_mean=False,with_std=True)

#variables transformées
Zcars = stds.fit_transform(cars)
print(numpy.std(Zcars,axis=0))

#local outlier factor
from sklearn.neighbors import LocalOutlierFactor
loc = LocalOutlierFactor(n_neighbors=3)

#calcul
loc.fit(Zcars)

#degré d'anormalité des observations
print(pandas.DataFrame({"Modele":cars.index,"NOF":loc.negative_outlier_factor_}).sort_values(by='NOF'))

#valeur seuil
print(loc.offset_)

#transformation des observations supplémentaires
Zcars_supp = stds.transform(cars_supp)
print(pandas.DataFrame(index=cars_supp.index,data=Zcars_supp,columns=cars_supp.columns))


#identification - patatras
loc.predict(Zcars_supp)

#instaciation avec le bon paramètre
loc_bis = LocalOutlierFactor(n_neighbors=3, novelty=True)

#apprentissage et identification
loc_bis.fit(Zcars)

#prédiction
loc_atyp = loc_bis.predict(Zcars_supp)
print(pandas.DataFrame({"Modele":cars_supp.index,"Atypique":loc_atyp}))

#afficher les véhicules parmi les références
tmp = pandas.DataFrame(data=Zcars,columns=cars.columns)
tmp = tmp.append(pandas.DataFrame(Zcars_supp,columns=cars.columns))
tmp['id'] = numpy.zeros(tmp.shape[0])
tmp.iloc[-4:-1,-1] = 1
tmp.iloc[-1,-1] = 2

#graphique
sns.pairplot(tmp,hue='id',palette={0:'silver',1:'green',2:'red'},diag_kind='hist')
